//	NPBJoystickDialog.cpp - Dialog for various joystick options.
//	--------------------------------------------------------------------------
//	This file is part of 'Niall's Pedal Board', a simple ASIO VST host.
//	Copyright (C) 2004-2005  Niall Moody
//	
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//	
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//	
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//	
//	--------------------------------------------------------------------------
//	
//	You can e-mail me at niallmoody@yahoo.co.uk if you want.
//	
//	--------------------------------------------------------------------------

#include "NPBJoystickDialog.h"
#include "NiallsTB.h"

NPBJoystickDialog::NPBJoystickDialog(NPBWindow *parent, Colour col):
DialogWindow(T("Joystick Options"), col)
{
	setVisible(true);

	NPBJoystickContent *cont = new NPBJoystickContent(parent, col);
	setContentComponent(cont);
}

//--------------------------------------------------------------------------
NPBJoystickContent::NPBJoystickContent(NPBWindow *parent, Colour col):
Component()
{
	String tempstr;

	this->parent = parent;

	addAndMakeVisible(smoothing =  new ToggleButton(T("Smooth Joystick Motion")));
	smoothing->setSize(50, 25);
	smoothing->changeWidthToFitText();
	smoothing->addActionListener(this);

	addAndMakeVisible(edLabel = new Label(T("edLabel"), T("Envelope Decrement: ")));
	addAndMakeVisible(envelopeDecrement = new TextEditor(T("Envelope Decrement")));
	tempstr << parent->getEnvelopeDecrement();
	envelopeDecrement->setText(tempstr);
	envelopeDecrement->setColours(Colours::black,
								  Colours::white,
								  Colour(0x401111ee),
								  Colours::grey);

	smoothing->setToggleState(parent->getJoystickSmoothing(), false);

	Colour tempc = col.darker(0.21f);
	Colour temp2 = col.darker(0.61f);
	addAndMakeVisible(ok = new NiallsTB(T("OK"), T("Yay!")));
	ok->setBackgroundColours(tempc, Colours::white);
	ok->addActionListener(this);

	addAndMakeVisible(cancel = new NiallsTB(T("Cancel"), T("Boo!")));
	cancel->setBackgroundColours(tempc, Colours::white);
	cancel->addActionListener(this);
}

void NPBJoystickContent::actionListenerCallback(const String& m)
{
	float tempf;
	if(m == ok->getName())
	{
		parent->setJoystickSmoothing(smoothing->getToggleState());
		tempf = envelopeDecrement->getText().getFloatValue();
		parent->setEnvelopeDecrement(tempf);
		((DialogWindow *)getParentComponent())->closeButtonPressed();
	}
	else if(m == cancel->getName())
		((DialogWindow *)getParentComponent())->closeButtonPressed();
}

void NPBJoystickContent::resized()
{
	smoothing->setTopLeftPosition(10, 10);
	edLabel->setBounds(10, 40, 150, 25);
	envelopeDecrement->setBounds(160, 40, 125, 25);

	ok->setBounds(10, (getHeight()-31), 100, 25);
	cancel->setBounds((getWidth()-110), (getHeight()-31), 100, 25);
}
