//	NPBPlugin.cpp - Vst plugin object, derived from
//				    NPBPluginBase.
//	--------------------------------------------------------------------------
//	This file is part of 'Niall's Pedal Board', a simple ASIO VST host.
//	Copyright (C) 2004-2005  Niall Moody
//	
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//	
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//	
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//	
//	--------------------------------------------------------------------------
//	
//	You can e-mail me at niallmoody@yahoo.co.uk if you want.
//	
//	--------------------------------------------------------------------------

#include "NPBPlugin.h"
#include "NiallsTB.h"

#ifndef NPBPLUGINORGANISER_H_
#include "NPBPluginOrganiser.h"
#endif

#ifndef NPBMAPPINGSDIALOG_H_
#include "NPBMappingsDialog.h"
#endif

long VSTCALLBACK host(AEffect *effect, long opcode, long index, long value, void *ptr, float opt);

Factory<NPBPluginBase, NPBPlugin> NPBPlugin::myFactory;

NPBPlugin::NPBPlugin(NPBPluginOrganiser *parent):
NPBPluginBase(parent),
envelope(0.0f),
envelopeDecrement(0.0001f)
{
	int i;
	String tempstr;

	PluginName = T("none");
	PluginHandle = 0;
	Plugin = 0;

	Editor = 0;
	GUI = 0;
	EditorOpen = false;

	Mix = 1.0f;// = fully wet

	midiNoteVal = 64;
	for(i=0;i<128;i++)
		noteOn[i] = false;
	wantsEvents = false;

	for(i=0;i<250;i++)
		pendingEvents[i] = false;
	numPendingEvents = 0;

	tempstr = ((NPBWindow *)Parent->getParentComponent())->getPluginPath();
	FileChooser dlg(T("Select a plugin (*.dll)"), File(tempstr), T("*.dll"));
	if(dlg.browseForFileToOpen())
		LoadPlugin(dlg.getResult());

	//Pass the VST Callback a pointer to our NPBPluginOrganiser.	
	host(Plugin,
		 -1,
		 0,
		 0,
		 static_cast<void *>(parent),
		 0);

	Colour tempc = Parent->getPlugButtonColour();
	Colour temp2 = tempc.darker(0.4f);
	addAndMakeVisible(Edit = new NiallsTB(T("Edit"), T("Open the plugin's editor")));
	Edit->setBackgroundColours(tempc, Colours::white);
	Edit->addActionListener(this);
	Edit->setSize(65, 25);

	addAndMakeVisible(Mappings = new NiallsTB(T("Mappings"), T("Edit the mappings for this plugin")));
	Mappings->setBackgroundColours(tempc, Colours::white);
	Mappings->addActionListener(this);
	Mappings->setSize(65, 25);

	addAndMakeVisible(Bypass = new NiallsTB(T("Bypass"), T("Bypass this plugin")));
	Bypass->setBackgroundColours(tempc, Colours::white);
	Bypass->addActionListener(this);
	Bypass->setSize(65, 25);

	setSize(75, 125);
}

NPBPlugin::NPBPlugin(NPBPluginOrganiser *parent, String filepath):NPBPluginBase(parent)
{
	int i;
	PluginName = T("none");
	PluginHandle = 0;
	Plugin = 0;

	Editor = 0;
	GUI = 0;
	EditorOpen = false;

	Mix = 1.0f;// = fully wet

	midiNoteVal = 64;
	for(i=0;i<128;i++)
		noteOn[i] = false;
	wantsEvents = false;

	for(i=0;i<250;i++)
		pendingEvents[i] = false;
	numPendingEvents = 0;

	LoadPlugin(File(filepath));

	//Pass the VST Callback a pointer to our NPBPluginOrganiser.	
	host(Plugin,
		 -1,
		 0,
		 0,
		 static_cast<void *>(parent),
		 0);

	Colour tempc = parent->getPlugButtonColour();
	Colour temp2 = tempc.darker(0.2f);
	addAndMakeVisible(Edit = new NiallsTB(T("Edit"), T("Open the plugin's editor")));
	Edit->setBackgroundColours(tempc, Colours::white);
	Edit->addActionListener(this);
	Edit->setSize(65, 25);

	addAndMakeVisible(Mappings = new NiallsTB(T("Mappings"), T("Edit the mappings for this plugin")));
	Mappings->setBackgroundColours(tempc, Colours::white);
	Mappings->addActionListener(this);
	Mappings->setSize(65, 25);

	addAndMakeVisible(Bypass = new NiallsTB(T("Bypass"), T("Bypass this plugin")));
	Bypass->setBackgroundColours(tempc, Colours::white);
	Bypass->addActionListener(this);
	Bypass->setSize(65, 25);

	setSize(75, 125);
}

NPBPlugin::~NPBPlugin()
{
	if(isTimerRunning())
		stopTimer();

	if(Editor)
	{
		Editor->closeButtonPressed();
	}
	else if(GUI)
	{
		GUI->closeButtonPressed();
	}

	if(Plugin)
	{
		Plugin->dispatcher(Plugin,effClose,0,0,NULL,0.0f);
		Process::freeDynamicLibrary(PluginHandle);
	}
}

void NPBPlugin::paint(Graphics& g)
{
	g.setColour(Colour::greyLevel(0.4f));
    RectangleList visible;
    getVisibleArea(visible, false);
    visible.fill(g);

	SolidColourBrush brush(Parent->getPlugColour());
	g.setBrush(&brush);
	g.fillAll();

	g.setColour (Colours::black.withAlpha (0.25f));
    g.drawRect (0, 0, getWidth(), getHeight());
	g.setColour (Colours::black.withAlpha (0.125f));
    g.drawRect (1, 1, (getWidth()-2), (getHeight()-2));

	//draw plugin's name
	g.setColour(Colour::greyLevel(0.45f));
    g.drawFittedText(PluginName,
                      5, 60,
                      getWidth() - 10, 25,
                      Justification::centred, 2);
}

void NPBPlugin::actionListenerCallback(const String& m)
{
	String temp;
	NPBWindow *window;

	if(m == Bypass->getName())
	{
		if(Bypass->getButtonText() == T("Bypass"))
			Bypass->setButtonText(T("Un-Bypass"));
		else
			Bypass->setButtonText(T("Bypass"));
	}
	else if(m == Edit->getName())
	{
		if(!EditorOpen)
		{
			EditorOpen = true;
			if((Plugin->flags&effFlagsHasEditor) != effFlagsHasEditor)
			{
				window = static_cast<NPBWindow *>(Parent->getParentComponent());
				Editor = new NPBPluginEditor(Plugin, this, window->getBackgroundColour());
				Editor->addToDesktop(windowHasDropShadow);
				Editor->centreWithSize(460, 345);
			}
			else
			{
				window = static_cast<NPBWindow *>(Parent->getParentComponent());
				GUI = new NPBPluginGUI(Plugin, this, window->getBackgroundColour());
				//GUI->addToDesktop(windowHasDropShadow);
				//GUI->setTopLeftPosition(250, 250);
			}
		}
		else
		{
			if(Editor)
				Editor->closeButtonPressed();
			else
				GUI->closeButtonPressed();
		}
	}
	else if(m == Mappings->getName())
	{
		window = static_cast<NPBWindow *>(Parent->getParentComponent());
		NPBMappingsDialog *dlg = new NPBMappingsDialog(this,
													   &MappingsArray,
													   window->getBackgroundColour());
		dlg->addToDesktop(windowHasDropShadow);

		dlg->centreWithSize(400, 300);
		dlg->enterModalState();
	}
}

void NPBPlugin::timerCallback()
{
	if(!Plugin->dispatcher(Plugin, effIdle, 0, 0, NULL, 0.0f))
		stopTimer();
}

void NPBPlugin::ProcessAudio(float **input, float **output, int numsamples)
{
	int i, j;
	float tempf;
	CriticalSection critSec;
	bool envDetectOn = false;

	if(Bypass->getButtonText() == T("Un-Bypass"))
	{
		for(i=0;i<NumChannels;i++)
		{
			for(j=0;j<numsamples;j++)
			{
				output[i][j] = input[i][j];
			}
		}
	}
	else if(Plugin) //**add in a short fade in/out for bypassing**
	{
		//If we've got mappings involving the envelope detector, pass them
		//onto the plugin.
		for(i=0;i<MappingsArray.size();i++)
		{
			if(MappingsArray[i]->JoyControl == evEnvelopeDetector)
			{
				envDetectOn = true;

				tempf = envelope;
				tempf *= (MappingsArray[i]->MaxVal - MappingsArray[i]->MinVal);
				tempf += MappingsArray[i]->MinVal; //this'll handle reversed ranges too

				handleMapping(i, tempf);
			}
		}

		//Pass any pending MIDI events to the plugin.
		if((wantsEvents) && (numPendingEvents > 0))
		{
			critSec.enter(); //Necessary to make this thread safe?

			j = 0;
			for(i=0;i<250;i++)
			{
				if(pendingEvents[i])
				{
					//If this event is pending, copy it to the event queue for the plugin.
					events.events[j] = &mainEvents[i];
					j++;
				}
				if(j >= numPendingEvents)
					break;
			}
			events.numEvents = numPendingEvents;

			//Send events to the plugin.
			Plugin->dispatcher(Plugin,
							   effProcessEvents,
							   0,
							   0,
							   reinterpret_cast<VstEvents *>(&events),
							   0);

			//Now that we're done, remove all pending events.
			for(i=0;i<250;i++)
			{
				if(pendingEvents[i])
					pendingEvents[i] = false;
			}
			numPendingEvents = 0;

			critSec.exit();
		}

		Plugin->processReplacing(Plugin, input, output, numsamples);
		if((Mix < 1.0f)||(envDetectOn))
		{
			for(i=0;i<NumChannels;i++)
			{
				for(j=0;j<numsamples;j++)
				{
					//Update envelope detector.
					if(i == 0)
					{
						if(envelope < static_cast<float>(fabs(input[i][j])))
							envelope = static_cast<float>(fabs(input[i][j]));
						else if(envelope > 0.0f)
							envelope -= envelopeDecrement;
					}
					//Mix output w/input.
					output[i][j] = (output[i][j]*Mix) + (input[i][j]*(1.0f-Mix));
				}
			}
		}
	}
}

void NPBPlugin::StartAudio()
{
	int tempint;

	tempint = ((NPBPluginOrganiser *)getParentComponent())->getBlockSize();
	Plugin->dispatcher(Plugin, effSetBlockSize, 0, tempint, NULL, 0.0f);
	Plugin->dispatcher(Plugin, effMainsChanged, 0, 1, NULL, 0);
}

void NPBPlugin::StopAudio()
{
	Plugin->dispatcher(Plugin, effMainsChanged, 0, 0, NULL, 0);
}

void NPBPlugin::resized()
{
    Edit->setTopLeftPosition(5, 5);
    Mappings->setTopLeftPosition(5, 30);
    Bypass->setTopLeftPosition(5, 95);
}

void NPBPlugin::mouseDown(const MouseEvent& e)
{
	if(e.mods.isRightButtonDown())
	{
		PopupMenu menu;
		menu.addItem(1, "Delete this Plugin");

		const int result = menu.show();

		switch(result)
		{
			case 1:
				NeedsDeleted = true; //**necessary for the plugin to be deleted**
				getParentComponent()->postCommandMessage(DELETE_THIS_PLUGIN);
				break;
		}
	}
}

void NPBPlugin::vstIdle()
{
	Plugin->dispatcher(Plugin, effIdle, 0, 0, NULL, 0.0f); //hmm...
}

void NPBPlugin::setMappings(OwnedArray<NPBMapping> *array)
{
	int i;
	NPBMapping *temp;

	MappingsArray.clear(true); //because the user may have removed some mappings in the dialog
	for(i=0;i<array->size();i++)
	{
		temp = new NPBMapping;
		temp->Index = (*array)[i]->Index;
		temp->JoyControl = (*array)[i]->JoyControl;
		temp->JoyIndex = (*array)[i]->JoyIndex;
		temp->MinVal = (*array)[i]->MinVal;
		temp->MaxVal = (*array)[i]->MaxVal;
		temp->PluginParameter = (*array)[i]->PluginParameter;
		temp->parameterType = (*array)[i]->parameterType;
		temp->MIDI = (*array)[i]->MIDI;
		temp->MIDINote = (*array)[i]->MIDINote;
		MappingsArray.add(temp);
	}
}

void NPBPlugin::getParameterName(long index, String& name)
{
	char tempstr[256];
	sprintf(tempstr, "");
	Plugin->dispatcher(Plugin, effGetParamName, index, 0, tempstr, 0.0f);
	name = tempstr;
}

int NPBPlugin::getNumJoysticks()
{
	return Organiser->getJoysticks().size();
}

void NPBPlugin::joystickCallback(JoystickEvent& joystate, int index)
{
	int i, j;
	float tempval;
	CriticalSection critSec;
	char tempch; //For creating the MIDI bytes.
	VstMidiEvent *tempEv;
	int tempint;

	for(i=0;i<MappingsArray.size();i++)
	{
		if(index != MappingsArray[i]->JoyIndex)
			continue;
		if(joystate.getType() != MappingsArray[i]->JoyControl)
			continue;
		if(joystate.getIndex() != MappingsArray[i]->Index)
			continue;

		if(joystate.getType() == evButton)
			tempval = (float)joystate.getVal();
		else if(joystate.getType() == evPOV)
			tempval = ((float)(joystate.getVal()+1))/35999.0f;
		else
		{
			tempval = ((float)(joystate.getVal()+32767))/65536.0f;
			if((joystate.getType() == evAxis)&&(joystate.getIndex() == 1))
				tempval = 1.0f - tempval;
		}

		tempval *= (MappingsArray[i]->MaxVal - MappingsArray[i]->MinVal);
		tempval += MappingsArray[i]->MinVal; //this'll handle reversed ranges too

		//If it's a VST parameter we're automating.
		if(MappingsArray[i]->parameterType)
		{
			if(MappingsArray[i]->PluginParameter < Plugin->numParams)
			{
				Plugin->setParameter(Plugin, MappingsArray[i]->PluginParameter, tempval);
				if(Editor)
					Editor->setParameter(MappingsArray[i]->PluginParameter, tempval);
			}
			else if(MappingsArray[i]->PluginParameter == Plugin->numParams)
			{
				if(tempval < 0.5f)
				{
					if(Bypass->getButtonText() == T("Bypass"))
						Bypass->setButtonText(T("Un-Bypass"));
				}
				else
				{
					if(Bypass->getButtonText() == T("Un-Bypass"))
						Bypass->setButtonText(T("Bypass"));
				}
			}
			else if(MappingsArray[i]->PluginParameter == (Plugin->numParams+1))
			{
				Mix = tempval;
				if(Editor)
					Editor->setMix(Mix);
				if(GUI)
					GUI->setMix(Mix);
			}
		}
		else if(wantsEvents) //If it's a MIDI message, and the plugin supports MIDI events.
		{
			if(MappingsArray[i]->MIDI == 128)
			{
				if(noteOn[MappingsArray[i]->MIDINote] && (tempval < 1))
				{
					//Send note off message.
					critSec.enter();
					for(j=0;j<250;j++)
					{
						if(!pendingEvents[j])
						{
							tempEv = reinterpret_cast<VstMidiEvent *>(&mainEvents[j]);
							tempEv->type = kVstMidiType;
							tempEv->byteSize = 24;
							tempEv->deltaFrames = 0;
							tempEv->flags = 0;
							tempEv->noteLength = 0;
							tempEv->noteOffset = 0;
							//Construct the correct MIDI bytes
							tempch = 0x00;
							tempch |= 0x80;
							tempEv->midiData[0] = tempch;
							tempch = MappingsArray[i]->MIDINote;
							tempEv->midiData[1] = tempch;
							tempch = 0;
							tempEv->midiData[2] = tempch;
							tempEv->detune = 0;
							tempEv->noteOffVelocity = 0;
							tempEv->reserved1 = 0;
							tempEv->reserved2 = 0;

							pendingEvents[j] = true;
							numPendingEvents++;
							break;
						}
					}
					critSec.exit();

					noteOn[MappingsArray[i]->MIDINote] = false;
				}
				else if(!noteOn[MappingsArray[i]->MIDINote])
				{
					//Send note on message.
					critSec.enter();
					for(j=0;j<250;j++)
					{
						if(!pendingEvents[j])
						{
							tempEv = reinterpret_cast<VstMidiEvent *>(&mainEvents[j]);
							tempEv->type = kVstMidiType;
							tempEv->byteSize = 24;
							tempEv->deltaFrames = 0;
							tempEv->flags = 0;
							tempEv->noteLength = 0;
							tempEv->noteOffset = 0;
							//Construct the correct MIDI bytes
							tempch = 0x00;
							tempch |= 0x90;
							tempEv->midiData[0] = tempch;
							tempch = MappingsArray[i]->MIDINote;
							tempEv->midiData[1] = tempch;
							tempch = static_cast<char>(tempval);
							tempEv->midiData[2] = tempch;
							tempEv->detune = 0;
							tempEv->noteOffVelocity = 0;
							tempEv->reserved1 = 0;
							tempEv->reserved2 = 0;

							pendingEvents[j] = true;
							numPendingEvents++;
							break;
						}
					}
					critSec.exit();

					noteOn[MappingsArray[i]->MIDINote] = true;
				}
			}
			else if(MappingsArray[i]->MIDI == 129)
			{
				//Send pitch bend message.
				critSec.enter();
				for(j=0;j<250;j++)
				{
					if(!pendingEvents[j])
					{
						tempval *= (65536.0f/(MappingsArray[i]->MinVal+MappingsArray[i]->MaxVal));
						tempint = static_cast<int>(tempval);

						tempEv = reinterpret_cast<VstMidiEvent *>(&mainEvents[j]);
						tempEv->type = kVstMidiType;
						tempEv->byteSize = 24;
						tempEv->deltaFrames = 0;
						tempEv->flags = 0;
						tempEv->noteLength = 0;
						tempEv->noteOffset = 0;
						//Construct the correct MIDI bytes
						tempch = 0x00;
						tempch |= 0xE0;
						tempEv->midiData[0] = tempch;
						tempch = tempint & 0x00FF;
						tempEv->midiData[1] = tempch;
						tempch = tempint >> 8;
						tempEv->midiData[2] = tempch;
						tempEv->detune = 0;
						tempEv->noteOffVelocity = 0;
						tempEv->reserved1 = 0;
						tempEv->reserved2 = 0;

						pendingEvents[j] = true;
						numPendingEvents++;
						break;
					}
				}
				critSec.exit();
			}
			else if(MappingsArray[i]->MIDI < 128)
			{
				//Send CC message.
				critSec.enter();
				for(j=0;j<250;j++)
				{
					if(!pendingEvents[j])
					{
						tempEv = reinterpret_cast<VstMidiEvent *>(&mainEvents[j]);
						tempEv->type = kVstMidiType;
						tempEv->byteSize = 24;
						tempEv->deltaFrames = 0;
						tempEv->flags = 0;
						tempEv->noteLength = 0;
						tempEv->noteOffset = 0;
						//Construct the correct MIDI bytes
						tempch = 0x00;
						tempch |= 0xB0;
						tempEv->midiData[0] = tempch;
						tempch = static_cast<char>(MappingsArray[i]->MIDI);
						tempEv->midiData[1] = tempch;
						tempch = (static_cast<char>(tempval) & 0x7F);
						tempEv->midiData[2] = tempch;
						tempEv->detune = 0;
						tempEv->noteOffVelocity = 0;
						tempEv->reserved1 = 0;
						tempEv->reserved2 = 0;

						pendingEvents[j] = true;
						numPendingEvents++;
						break;
					}
				}
				critSec.exit();
			}
		}
	}
}

float NPBPlugin::getParameter(long index)
{
	if(index < Plugin->numParams)
		return Plugin->getParameter(Plugin, index);
	else if(index == Plugin->numParams)
	{
		if(Bypass->getButtonText() == "Bypass")
			return 0.0f;
		else
			return 1.0f;
	}
	else
		return Mix;
}

void NPBPlugin::setParameter(long index, float val)
{
	if(index < Plugin->numParams)
	{
		Plugin->setParameter(Plugin, index, val);
		if(Editor)
			Editor->setParameter(index, val);
	}
	else if(index == Plugin->numParams)
	{
		if(val < 0.5f)
			Bypass->setButtonText(T("Bypass"));
		else
			Bypass->setButtonText(T("Un-Bypass"));
	}
	else
	{
		Mix = val;
		if(Editor)
			Editor->setMix(Mix);
		else if(GUI)
			GUI->setMix(Mix);
	}
}

void NPBPlugin::addMIDIMessage(const MidiMessage& message)
{
	int i;
	float tempval;
	VstMidiEvent *tempEv;
	CriticalSection critSec;

	if(wantsEvents)
	{
		if(acceptMIDI)
		{
			critSec.enter();
			for(i=0;i<250;i++)
			{
				if(!pendingEvents[i])
				{
					tempEv = reinterpret_cast<VstMidiEvent *>(&mainEvents[i]);
					tempEv->type = kVstMidiType;
					tempEv->byteSize = 24;
					tempEv->deltaFrames = 0;
					tempEv->flags = 0;
					tempEv->noteLength = 0;
					tempEv->noteOffset = 0;
					tempEv->midiData[0] = message.getRawData()[0];
					tempEv->midiData[1] = message.getRawData()[1];
					tempEv->midiData[2] = message.getRawData()[2];
					tempEv->detune = 0;
					tempEv->noteOffVelocity = 0;
					tempEv->reserved1 = 0;
					tempEv->reserved2 = 0;

					pendingEvents[i] = true;
					numPendingEvents++;
					break;
				}
			}
			critSec.exit();
		}

		//Because acceptMIDI only applies to the above MIDI messages.
		switch(message.getRawData()[0] & 0xF0)
		{
			case 0x90: //NoteOn
				for(i=0;i<MappingsArray.size();i++)
				{
					if(MappingsArray[i]->JoyControl == evMIDINoteVal)
					{
						//i.e. NoteOn, not cheap NoteOff
						if((MappingsArray[i]->Index)&&
						   ((message.getRawData()[2]&0x7F) > 0))
						{
							tempval = static_cast<float>(message.getRawData()[1]&0x7F);
							tempval *= (MappingsArray[i]->MaxVal - MappingsArray[i]->MinVal);
							tempval += MappingsArray[i]->MinVal;
							tempval /= 127.0f;

							if(MappingsArray[i]->parameterType)
							{
								if(MappingsArray[i]->PluginParameter < Plugin->numParams)
								{
									Plugin->setParameter(Plugin, MappingsArray[i]->PluginParameter, tempval);
									if(Editor)
										Editor->setParameter(MappingsArray[i]->PluginParameter, tempval);
								}
								else if(MappingsArray[i]->PluginParameter == Plugin->numParams)
								{
									if(tempval < 0.5f)
									{
										if(Bypass->getButtonText() == T("Bypass"))
											Bypass->setButtonText(T("Un-Bypass"));
									}
									else
									{
										if(Bypass->getButtonText() == T("Un-Bypass"))
											Bypass->setButtonText(T("Bypass"));
									}
								}
								else if(MappingsArray[i]->PluginParameter == (Plugin->numParams+1))
								{
									Mix = tempval;
									if(Editor)
										Editor->setMix(Mix);
									if(GUI)
										GUI->setMix(Mix);
								}
							}
						}
						//It's a NoteOff.
						else if((!MappingsArray[i]->Index)&&
							    ((message.getRawData()[2]&0x7F) == 0))
						{
							tempval = static_cast<float>(message.getRawData()[1]&0x7F);
							tempval *= (MappingsArray[i]->MaxVal - MappingsArray[i]->MinVal);
							tempval += MappingsArray[i]->MinVal;
							tempval /= 127.0f;

							if(MappingsArray[i]->parameterType)
							{
								if(MappingsArray[i]->PluginParameter < Plugin->numParams)
								{
									Plugin->setParameter(Plugin, MappingsArray[i]->PluginParameter, tempval);
									if(Editor)
										Editor->setParameter(MappingsArray[i]->PluginParameter, tempval);
								}
								else if(MappingsArray[i]->PluginParameter == Plugin->numParams)
								{
									if(tempval < 0.5f)
									{
										if(Bypass->getButtonText() == T("Bypass"))
											Bypass->setButtonText(T("Un-Bypass"));
									}
									else
									{
										if(Bypass->getButtonText() == T("Un-Bypass"))
											Bypass->setButtonText(T("Bypass"));
									}
								}
								else if(MappingsArray[i]->PluginParameter == (Plugin->numParams+1))
								{
									Mix = tempval;
									if(Editor)
										Editor->setMix(Mix);
									if(GUI)
										GUI->setMix(Mix);
								}
							}
						}
					}
					//If it's a velocity event we're looking for.
					else if(MappingsArray[i]->JoyControl == evMIDIVelocity)
					{
						tempval = static_cast<float>(message.getRawData()[2]&0x7F);
						tempval *= (MappingsArray[i]->MaxVal - MappingsArray[i]->MinVal);
						tempval += MappingsArray[i]->MinVal;
						tempval /= 127.0f;

						if(MappingsArray[i]->parameterType)
						{
							if(MappingsArray[i]->PluginParameter < Plugin->numParams)
							{
								Plugin->setParameter(Plugin, MappingsArray[i]->PluginParameter, tempval);
								if(Editor)
									Editor->setParameter(MappingsArray[i]->PluginParameter, tempval);
							}
							else if(MappingsArray[i]->PluginParameter == Plugin->numParams)
							{
								if(tempval < 0.5f)
								{
									if(Bypass->getButtonText() == T("Bypass"))
										Bypass->setButtonText(T("Un-Bypass"));
								}
								else
								{
									if(Bypass->getButtonText() == T("Un-Bypass"))
										Bypass->setButtonText(T("Bypass"));
								}
							}
							else if(MappingsArray[i]->PluginParameter == (Plugin->numParams+1))
							{
								Mix = tempval;
								if(Editor)
									Editor->setMix(Mix);
								if(GUI)
									GUI->setMix(Mix);
							}
						}
					}
				}
				break;
			case 0x80: //NoteOff
				for(i=0;i<MappingsArray.size();i++)
				{
					if(MappingsArray[i]->JoyControl == evMIDINoteVal)
					{
						//It's a NoteOff.
						if(!MappingsArray[i]->Index)
						{
							tempval = static_cast<float>(message.getRawData()[1]&0x7F);
							tempval *= (MappingsArray[i]->MaxVal - MappingsArray[i]->MinVal);
							tempval += MappingsArray[i]->MinVal;
							tempval /= 127.0f;

							if(MappingsArray[i]->parameterType)
							{
								if(MappingsArray[i]->PluginParameter < Plugin->numParams)
								{
									Plugin->setParameter(Plugin, MappingsArray[i]->PluginParameter, tempval);
									if(Editor)
										Editor->setParameter(MappingsArray[i]->PluginParameter, tempval);
								}
								else if(MappingsArray[i]->PluginParameter == Plugin->numParams)
								{
									if(tempval < 0.5f)
									{
										if(Bypass->getButtonText() == T("Bypass"))
											Bypass->setButtonText(T("Un-Bypass"));
									}
									else
									{
										if(Bypass->getButtonText() == T("Un-Bypass"))
											Bypass->setButtonText(T("Bypass"));
									}
								}
								else if(MappingsArray[i]->PluginParameter == (Plugin->numParams+1))
								{
									Mix = tempval;
									if(Editor)
										Editor->setMix(Mix);
									if(GUI)
										GUI->setMix(Mix);
								}
							}
						}
					}
					//If it's a velocity event we're looking for.
					else if(MappingsArray[i]->JoyControl == evMIDIVelocity)
					{
						tempval = 0.0f;
						tempval *= (MappingsArray[i]->MaxVal - MappingsArray[i]->MinVal);
						tempval += MappingsArray[i]->MinVal;
						tempval /= 127.0f;

						if(MappingsArray[i]->parameterType)
						{
							if(MappingsArray[i]->PluginParameter < Plugin->numParams)
							{
								Plugin->setParameter(Plugin, MappingsArray[i]->PluginParameter, tempval);
								if(Editor)
									Editor->setParameter(MappingsArray[i]->PluginParameter, tempval);
							}
							else if(MappingsArray[i]->PluginParameter == Plugin->numParams)
							{
								if(tempval < 0.5f)
								{
									if(Bypass->getButtonText() == T("Bypass"))
										Bypass->setButtonText(T("Un-Bypass"));
								}
								else
								{
									if(Bypass->getButtonText() == T("Un-Bypass"))
										Bypass->setButtonText(T("Bypass"));
								}
							}
							else if(MappingsArray[i]->PluginParameter == (Plugin->numParams+1))
							{
								Mix = tempval;
								if(Editor)
									Editor->setMix(Mix);
								if(GUI)
									GUI->setMix(Mix);
							}
						}
					}
				}
				break;
			case 0xB0: //CC
				for(i=0;i<MappingsArray.size();i++)
				{
					if(MappingsArray[i]->JoyControl == evMIDICC)
					{
						if(MappingsArray[i]->Index == (message.getRawData()[1]&0x7F))
						{
							tempval = static_cast<float>(message.getRawData()[2]&0x7F);
							tempval *= (MappingsArray[i]->MaxVal - MappingsArray[i]->MinVal);
							tempval += MappingsArray[i]->MinVal;
							tempval /= 127.0f;

							if(MappingsArray[i]->parameterType)
							{
								if(MappingsArray[i]->PluginParameter < Plugin->numParams)
								{
									Plugin->setParameter(Plugin, MappingsArray[i]->PluginParameter, tempval);
									if(Editor)
										Editor->setParameter(MappingsArray[i]->PluginParameter, tempval);
								}
								else if(MappingsArray[i]->PluginParameter == Plugin->numParams)
								{
									if(tempval < 0.5f)
									{
										if(Bypass->getButtonText() == T("Bypass"))
											Bypass->setButtonText(T("Un-Bypass"));
									}
									else
									{
										if(Bypass->getButtonText() == T("Un-Bypass"))
											Bypass->setButtonText(T("Bypass"));
									}
								}
								else if(MappingsArray[i]->PluginParameter == (Plugin->numParams+1))
								{
									Mix = tempval;
									if(Editor)
										Editor->setMix(Mix);
									if(GUI)
										GUI->setMix(Mix);
								}
							}
						}
					}
				}
				break;
		}
	}
}

void NPBPlugin::LoadPlugin(File plugin)
{
	String tempstr2;
	int tempint;

	PluginHandle = Process::loadDynamicLibrary(plugin.getFullPathName());
	FilePath = plugin.getFullPathName();

	tempstr2 = plugin.getFullPathName();
	tempint = tempstr2.lastIndexOfChar(File::separator);
	tempstr2 = tempstr2.dropLastCharacters(tempstr2.length()-(tempint+1));
	((NPBWindow *)Parent->getParentComponent())->setPluginPath(tempstr2);

	if(PluginHandle)
	{
		AEffect* (/*__stdcall*/* getNewPlugInstance)(audioMasterCallback);
		getNewPlugInstance=(AEffect*(/*__stdcall*/*)(audioMasterCallback))Process::getProcedureEntryPoint(PluginHandle, "main");

		if(getNewPlugInstance)
		{
			Plugin = getNewPlugInstance(host);
			if(Plugin)
			{
				if(Plugin->magic==kEffectMagic)
				{
				}
				else
				{
					AlertWindow error(T("Plugin Error"), T("Not a valid plugin."), AlertWindow::WarningIcon);
					error.addButton(T("OK"), 0);
					error.runModalLoop();
					Process::freeDynamicLibrary(PluginHandle);
					return;
				}
			}
			else
			{
				AlertWindow error(T("Plugin Error"), T("Couldn't instantiate the plugin."), AlertWindow::WarningIcon);
				error.addButton(T("OK"), 0);
				error.runModalLoop();
				Process::freeDynamicLibrary(PluginHandle);
				return;
			}
		}
		else
		{
			AlertWindow error(T("Plugin Error"), T("Couldn't find the plugin's main() function."), AlertWindow::WarningIcon);
			error.addButton(T("OK"), 0);
			error.runModalLoop();
			Process::freeDynamicLibrary(PluginHandle);
			return;
		}
	}
	else
	{
		AlertWindow error(T("Plugin Error"), T("Couldn't load the plugin."), AlertWindow::WarningIcon);
		error.addButton(T("OK"), 0);
		error.runModalLoop();
		return;
	}

	//Get the plugin's name
	char tempstr[256];
	sprintf(tempstr, "");
	Plugin->dispatcher(Plugin, effGetEffectName, 0, 0, tempstr, 0.0f);
	PluginName = tempstr;

	//Set samplerate **change this from a hard-coded value**
	Plugin->dispatcher(Plugin, effSetSampleRate, 0, 0, NULL, 44100.0f);

	//open the plugin
	Plugin->dispatcher(Plugin, effOpen, 0, 0, NULL, 0.0f);

	//
}

void NPBPlugin::handleMapping(long mappingIndex, float val)
{
	int j;
	CriticalSection critSec;
	char tempch; //For creating the MIDI bytes.
	VstMidiEvent *tempEv;
	int tempint;

	//If it's a VST parameter we're automating.
	if(MappingsArray[mappingIndex]->parameterType)
	{
		if(MappingsArray[mappingIndex]->PluginParameter < Plugin->numParams)
			Plugin->setParameter(Plugin, MappingsArray[mappingIndex]->PluginParameter, val);
		else if(MappingsArray[mappingIndex]->PluginParameter == Plugin->numParams)
		{
			if(val < 0.5f)
			{
				if(Bypass->getButtonText() == T("Bypass"))
					Bypass->setButtonText(T("Un-Bypass"));
			}
			else
			{
				if(Bypass->getButtonText() == T("Un-Bypass"))
					Bypass->setButtonText(T("Bypass"));
			}
		}
		else if(MappingsArray[mappingIndex]->PluginParameter == (Plugin->numParams+1))
			Mix = val;
	}
	else if(wantsEvents) //If it's a MIDI message, and the plugin supports MIDI events.
	{
		if(MappingsArray[mappingIndex]->MIDI == 128)
		{
			if(noteOn[MappingsArray[mappingIndex]->MIDINote] && (val < 1.0f))
			{
				//Send note off message.
				critSec.enter();
				for(j=0;j<250;j++)
				{
					if(!pendingEvents[j])
					{
						tempEv = reinterpret_cast<VstMidiEvent *>(&mainEvents[j]);
						tempEv->type = kVstMidiType;
						tempEv->byteSize = 24;
						tempEv->deltaFrames = 0;
						tempEv->flags = 0;
						tempEv->noteLength = 0;
						tempEv->noteOffset = 0;
						//Construct the correct MIDI bytes
						tempch = 0x00;
						tempch |= 0x80;
						tempEv->midiData[0] = tempch;
						tempch = MappingsArray[mappingIndex]->MIDINote;
						tempEv->midiData[1] = tempch;
						tempch = 0;
						tempEv->midiData[2] = tempch;
						tempEv->detune = 0;
						tempEv->noteOffVelocity = 0;
						tempEv->reserved1 = 0;
						tempEv->reserved2 = 0;

						pendingEvents[j] = true;
						numPendingEvents++;
						break;
					}
				}
				critSec.exit();
				noteOn[MappingsArray[mappingIndex]->MIDINote] = false;
			}
			else if(!noteOn[MappingsArray[mappingIndex]->MIDINote])
			{
				//Send note on message.
				critSec.enter();
				for(j=0;j<250;j++)
				{
					if(!pendingEvents[j])
					{
						tempEv = reinterpret_cast<VstMidiEvent *>(&mainEvents[j]);
						tempEv->type = kVstMidiType;
						tempEv->byteSize = 24;
						tempEv->deltaFrames = 0;
						tempEv->flags = 0;
						tempEv->noteLength = 0;
						tempEv->noteOffset = 0;
						//Construct the correct MIDI bytes
						tempch = 0x00;
						tempch |= 0x90;
						tempEv->midiData[0] = tempch;
						tempch = MappingsArray[mappingIndex]->MIDINote;
						tempEv->midiData[1] = tempch;
						tempch = static_cast<char>(val);
						tempEv->midiData[2] = tempch;
						tempEv->detune = 0;
						tempEv->noteOffVelocity = 0;
						tempEv->reserved1 = 0;
						tempEv->reserved2 = 0;

						pendingEvents[j] = true;
						numPendingEvents++;
						break;
					}
				}
				critSec.exit();
				noteOn[MappingsArray[mappingIndex]->MIDINote] = true;
			}
		}
		else if(MappingsArray[mappingIndex]->MIDI == 129)
		{
			//Send pitch bend message.
			critSec.enter();
			for(j=0;j<250;j++)
			{
				if(!pendingEvents[j])
				{
					val *= (65536.0f/(MappingsArray[mappingIndex]->MinVal + MappingsArray[mappingIndex]->MaxVal));
					tempint = static_cast<int>(val);
					tempEv = reinterpret_cast<VstMidiEvent *>(&mainEvents[j]);
					tempEv->type = kVstMidiType;
					tempEv->byteSize = 24;
					tempEv->deltaFrames = 0;
					tempEv->flags = 0;
					tempEv->noteLength = 0;
					tempEv->noteOffset = 0;
					//Construct the correct MIDI bytes
					tempch = 0x00;
					tempch |= 0xE0;
					tempEv->midiData[0] = tempch;
					tempch = tempint & 0x00FF;
					tempEv->midiData[1] = tempch;
					tempch = tempint >> 8;
					tempEv->midiData[2] = tempch;
					tempEv->detune = 0;
					tempEv->noteOffVelocity = 0;
					tempEv->reserved1 = 0;
					tempEv->reserved2 = 0;

					pendingEvents[j] = true;
					numPendingEvents++;
					break;
				}
			}
			critSec.exit();
		}
		else if(MappingsArray[mappingIndex]->MIDI < 128)
		{
			//Send CC message.
			critSec.enter();
			for(j=0;j<250;j++)
			{
				if(!pendingEvents[j])
				{
					tempEv = reinterpret_cast<VstMidiEvent *>(&mainEvents[j]);
					tempEv->type = kVstMidiType;
					tempEv->byteSize = 24;
					tempEv->deltaFrames = 0;
					tempEv->flags = 0;
					tempEv->noteLength = 0;
					tempEv->noteOffset = 0;
					//Construct the correct MIDI bytes
					tempch = 0x00;
					tempch |= 0xB0;
					tempEv->midiData[0] = tempch;
					tempch = static_cast<char>(MappingsArray[mappingIndex]->MIDI);
					tempEv->midiData[1] = tempch;
					tempch = (static_cast<char>(val) & 0x7F);
					tempEv->midiData[2] = tempch;
					tempEv->detune = 0;
					tempEv->noteOffVelocity = 0;
					tempEv->reserved1 = 0;
					tempEv->reserved2 = 0;

					pendingEvents[j] = true;
					numPendingEvents++;
					break;
				}
			}
			critSec.exit();
		}
	}
}

long VSTCALLBACK host(AEffect *effect, long opcode, long index, long value, void *ptr, float opt)
{
	static NPBPluginOrganiser *Organiser = 0;
	NPBPlugin *tempPlug;
	long retval = 0;

	switch (opcode)
	{
		case -1: //I'm hoping this is ok...
			Organiser = static_cast<NPBPluginOrganiser *>(ptr);
			break;
		case audioMasterVersion:
			retval = 2300; //2300 = 2.3
			break;
		case audioMasterAutomate:
			//Called when the plugin calls setParameterAutomated.
			break;
		case audioMasterCurrentId:
			//Wants the unique ID of a plugin that's currently loading.
			break;
		case audioMasterIdle:
			//Should call the application's idle routine (including effEditIdle).
			if(Organiser)
			{
				Organiser->vstIdle();
			}
			break;
		case audioMasterPinConnected:
			//Checks if a pin has been connected as input or output.
			break;
		case audioMasterWantMidi:
			//The plugin wants MIDI.
			if(Organiser)
			{
				tempPlug = static_cast<NPBPlugin *>(Organiser->findPlugin(effect));
				if(tempPlug)
					tempPlug->setWantsEvents();
			}
			break;
		case audioMasterGetTime:
			//Return is a VstTimeInfo structure, or 0 if not supported.
			break;
		case audioMasterProcessEvents:
			//Plugin sends us VstEvents to be processed?
			retval = 1; //1 = no error
			break;
		case audioMasterSetTime:
			//Ignore?
			break;
		case audioMasterTempoAt:
			//Plugin wants the tempo at sampleframe <value>
			if(Organiser)
				return static_cast<long>(Organiser->getBpm()*10000.0f);
			break;
		case audioMasterGetNumAutomatableParameters:
			//The plugin asks how many parameters may be automated?
			break;
		case audioMasterGetParameterQuantization:
			//Automation quantisation.
			retval = 1; //1 = full single float precision
			break;
		case audioMasterIOChanged:
			//Plugin signals that it's inputs/outputs have changed
			//0 = error, 1 = ok
			break;
		case audioMasterNeedIdle:
			//The plugin needs idle calls (outside it's editor's idle calls).
			//Need to call effIdle
			retval = 1;
			//Call NPBPluginOrganiser::startIdleCalls(AEffect *) here.
			break;
		case audioMasterSizeWindow:
			//The plugin wants to resize it's window?
			break;
		case audioMasterGetSampleRate:
			//Must call plugin->setSamplerate in response
			//effect->dispatcher(effect,effSetSampleRate,0,0,NULL,samplerate);
			break;
		case audioMasterGetBlockSize:
			//Must call plugin->setBlockSize in response
			//effect->dispatcher(effect,effSetBlockSize,0,blocksize,NULL,0.0f);
			break;
		case audioMasterGetInputLatency:
			//Return input latency.
			break;
		case audioMasterGetOutputLatency:
			//Return input latency.
			break;
		case audioMasterGetPreviousPlug:
			//Return AEffect *, or NULL
			retval = NULL;
			break;
		case audioMasterGetNextPlug:
			//Return AEffect *, or NULL
			retval = NULL;
			break;
		case audioMasterWillReplaceOrAccumulate:
			//0=not supported, 1=replace, 2=accumulate
			retval = 1;
			break;
		case audioMasterGetCurrentProcessLevel:
			break;
		case audioMasterGetAutomationState:
			break;
		case audioMasterGetVendorString:
			//<ptr> contains a char * to be populated.
			strcpy((char *)ptr, "Niall Moody");
			retval = 1;
			break;
		case audioMasterGetProductString:
			//<ptr> contains a char * to be populated.
			strcpy((char *)ptr, "Niall's Pedal Board");
			retval = 1;
			break;
		case audioMasterGetVendorVersion:
			//Return version.
			retval = 30; //030 = v0.3
			break;
		case audioMasterVendorSpecific:
			//Vendor specific call.  We're going to use it to set a pointer to our NPBPluginOrganiser.
			if(!Organiser)
				Organiser = (NPBPluginOrganiser *)ptr;
			break;
		case audioMasterSetIcon:
			//Ignore this.
			break;
		case audioMasterCanDo:
			//<ptr> = CanDo string, possible values are:
			//"sendVstEvents"
			//"sendVstMidiEvent"
			//"sendVstTimeInfo"
			//"receiveVstEvents"
			//"receiveVstMidiEvent"
			//"receiveVstTimeInfo"
			//"reportConnectionChanges"
			//"acceptIOChanges"
			//"sizeWindow"
			//"asyncProcessing" - for external hardware type stuff
			//"offline"
			//"supplyIdle"
			//"supportShell"
			//"openFileSelector"
			//"editFile" - Is this to allow editors defined by xml?  That could be pretty cool, I should support this...
			//"closeFileSelector"
			//"startStopProcess" - huh?
			if(strcmp((char *)ptr, "supplyIdle")==0)
				retval = 1; //cando, 0=nocando
			else if(strcmp((char *)ptr, "openFileSelector")==0)
				retval = 0;
			else if(strcmp((char *)ptr, "closeFileSelector")==0)
				retval = 0;
			else
				retval = -1;
			break;
		case audioMasterGetLanguage:
			//Get the host's language from an enum in aeffectx.h
			retval = kVstLangEnglish;
			break;
		case audioMasterGetDirectory:
			//Get the plugin's directory (FSSpec on Mac)
			//return as char *?
			break;
		case audioMasterUpdateDisplay:
			//Something has changed, update host's NPBPluginEditor window
			break;
		case audioMasterBeginEdit:
			//Start automation session.
			break;
		case audioMasterEndEdit:
			//Stop automation session.
			break;
		case audioMasterOpenFileSelector:
			//Plugin wants you to open a file selector, with a VstFileSelect* structure in <ptr>.
			break;
		case audioMasterCloseFileSelector:
			//Must be called after OpenFileSelector, fill <ptr> with the relevant VstFileSelect details.
			retval = -1;
			break;
		case audioMasterEditFile:
			//Opens an editor, using xml (in <ptr>) to define it.  Could be pretty cool...
			break;
		case audioMasterGetChunkFile:
			//Get the path of the currently-loading chunk file.
			break;
		default:
			retval = 0;
			break;
	}
	return retval;
}
