//	NPBVSTDialog.cpp - Dialog for various VST-specific options.
//	--------------------------------------------------------------------------
//	This file is part of 'Niall's Pedal Board', a simple ASIO VST host.
//	Copyright (C) 2004-2005  Niall Moody
//	
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//	
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//	
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//	
//	--------------------------------------------------------------------------
//	
//	You can e-mail me at niallmoody@yahoo.co.uk if you want.
//	
//	--------------------------------------------------------------------------

#include "NPBVSTDialog.h"

NPBVSTDialog::NPBVSTDialog(NPBWindow *parent, Colour col):
DialogWindow(T("VST Options"), col)
{
	setVisible(true);

	NPBVSTContent *cont = new NPBVSTContent(parent, col);
	setContentComponent(cont);
}

//--------------------------------------------------------------------------
NPBVSTContent::NPBVSTContent(NPBWindow *parent, Colour col):
Component()
{
	String tempstr;

	this->parent = parent;

	addAndMakeVisible(bpmLabel = new Label(T("bpmLabel"), T("BPM: ")));
	addAndMakeVisible(bpm = new TextEditor(T("bpm")));
	tempstr << parent->getBpm();
	bpm->setText(tempstr);
	bpm->setColours(Colours::black,
					Colours::white,
					Colour(0x401111ee),
					Colours::grey);

	Colour tempc = col.darker(0.21f);
	Colour temp2 = col.darker(0.61f);
	addAndMakeVisible(ok = new TextButton(T("OK"), T("Yay!")));
	ok->setBackgroundColours(tempc, Colours::white);
	ok->addActionListener(this);

	addAndMakeVisible(cancel = new TextButton(T("Cancel"), T("Boo!")));
	cancel->setBackgroundColours(tempc, Colours::white);
	cancel->addActionListener(this);
}

void NPBVSTContent::actionListenerCallback(const String& m)
{
	float tempf;
	if(m == ok->getName())
	{
		tempf = bpm->getText().getFloatValue();
		parent->setBpm(tempf);
		((DialogWindow *)getParentComponent())->closeButtonPressed();
	}
	else if(m == cancel->getName())
		((DialogWindow *)getParentComponent())->closeButtonPressed();
}

void NPBVSTContent::resized()
{
	bpmLabel->setBounds(10, 10, 45, 25);
	bpm->setBounds(50, 10, 75, 25);

	ok->setBounds(10, (getHeight()-31), 100, 25);
	cancel->setBounds((getWidth()-110), (getHeight()-31), 100, 25);
}
