///	MainWindow.h - 'Dummy' subclass of FactoryObject, to set various aspects
///				    of the main window.
///	--------------------------------------------------------------------------
///	This file is part of 'Niall's Pedal Board', a simple ASIO VST host.
///	Copyright (C) 2005  Niall Moody
///	
///	This program is free software; you can redistribute it and/or modify
///	it under the terms of the GNU General Public License as published by
///	the Free Software Foundation; either version 2 of the License, or
///	(at your option) any later version.
///	
///	This program is distributed in the hope that it will be useful,
///	but WITHOUT ANY WARRANTY; without even the implied warranty of
///	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
///	GNU General Public License for more details.
///	
///	You should have received a copy of the GNU General Public License
///	along with this program; if not, write to the Free Software
///	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
///	
///	--------------------------------------------------------------------------
///	
///	You can e-mail me at niallmoody@yahoo.co.uk if you want.
///	
///	--------------------------------------------------------------------------

#ifndef MAINWINDOW_H_
#define MAINWINDOW_H_

#include "../FactoryObject.h"

///	FactoryObject for the main window.
class MainWindow : public FactoryObject
{
  public:
	MainWindow(ComponentFactory *factory, Component **windowComponent);
	~MainWindow() {};

	/// Doesn't do anything, because the window's already been created...
	void create() {};

	static void setResizeable(bool val);
	static void setMaximise(bool val);
	static void setColourRed(int val);
	static void setColourGreen(int val);
	static void setColourBlue(int val);
	static void setDropShadow(bool val);
	static void setCentre(bool val);
	static void setTitle(String val);
	static void setTitleColourRed(int val);
	static void setTitleColourGreen(int val);
	static void setTitleColourBlue(int val);
	static void setTitleX(int val);
	static void setTitleY(int val);
	static void setLookAndFeel(int val);

	//  Shouldn't need to override these methods for any other subclass.
	void setX(int val);
	void setY(int val);
	void setXRel(double val);
	void setYRel(double val);
	void setWidth(int val);
	void setHeight(int val);
	void setWidthRel(double val);
	void setHeightRel(double val);
};

#endif
